@@ -19,7 +19,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  */
 
@@ -178,7 +178,6 @@ typedef struct {
 	const GodDefaultAttributes *default_attributes;
 } DrawTextContext;
 
-#ifdef PANGO_HACK
 static gboolean
 make_absolute (PangoAttribute *attr, gpointer user_data)
 {
@@ -191,7 +190,6 @@ make_absolute (PangoAttribute *attr, gpointer user_data)
 	}
 	return FALSE;
 }
-#endif
 
 static void
 draw_text (GodTextModel *text,
@@ -210,8 +208,9 @@ draw_text (GodTextModel *text,
 	const GodParagraphAttributes *default_para_attributes;
 	gunichar bullet_character = 0;
 	double bullet_indent = 0;
-	double bullet_size = 1.0;
+	double bullet_size = 0;
 	char *bullet_family = NULL;
+	gboolean bullet_on = FALSE;
 	PangoFontDescription *bullet_desc;
 	PangoAttrIterator *attr_iterator;
 
@@ -227,6 +226,7 @@ draw_text (GodTextModel *text,
 				      "bullet_indent", &bullet_indent,
 				      "bullet_size", &bullet_size,
 				      "bullet_family", &bullet_family,
+				      "bullet_on", &bullet_on,
 				      NULL);
 		}
 	}
@@ -270,6 +270,11 @@ draw_text (GodTextModel *text,
 				      "bullet_family", &bullet_family,
 				      NULL);
 		}
+		if (flags & GOD_PARAGRAPH_ATTRIBUTES_FLAGS_BULLET_ON) {
+			g_object_get (paragraph->para_attributes,
+				      "bullet_on", &bullet_on,
+				      NULL);
+		}
 	}
 	draw_context->y_ofs += space_before;
 
@@ -302,9 +307,7 @@ draw_text (GodTextModel *text,
 			pango_attr_list_insert_before (attributes, attr);
 		}
 	}
-#ifdef PANGO_HACK
 	pango_attr_list_filter (attributes, make_absolute, draw_context);
-#endif
 	pango_layout_set_attributes (layout, attributes);
 	attr_iterator = pango_attr_list_get_iterator (attributes);
 	bullet_desc = pango_font_description_new ();
@@ -325,7 +328,8 @@ draw_text (GodTextModel *text,
 	if (bullet_character != 0 &&
 	    bullet_character != 0xe011 &&
 	    bullet_size != 0 &&
-	    bullet_family != NULL) {
+	    bullet_family != NULL &&
+	    bullet_on) {
 		char utf8[7];
 		int length;
 		layout = pango_layout_new (gdk_pango_context_get_for_screen (gdk_screen_get_default()));
@@ -333,6 +337,11 @@ draw_text (GodTextModel *text,
 		length = g_unichar_to_utf8 (bullet_character, utf8);
 		pango_layout_set_text (layout, utf8, length);
 		pango_layout_set_auto_dir (layout, FALSE);
+#if 0
+		pango_font_description_set_absolute_size (bullet_desc, 
+							  GO_PT_TO_UN ((go_unit_t) bullet_size * PANGO_SCALE) /
+							  draw_context->renderer->priv->y_units_per_pixel);
+#endif
 		pango_font_description_set_size (bullet_desc, pango_font_description_get_size (bullet_desc) * sqrt (bullet_size));
 		pango_font_description_set_family (bullet_desc, bullet_family);
 		pango_layout_set_font_description (layout, bullet_desc);
@@ -349,14 +358,6 @@ draw_text (GodTextModel *text,
 
 	draw_context->y_ofs += height * draw_context->renderer->priv->y_units_per_pixel / PANGO_SCALE;
 	draw_context->y_ofs += space_after;
-
-#if 0
-	g_print ("space before: %f\n", space_before);
-	g_print ("space after: %f\n", space_after);
-	g_print ("indent: %f\n", indent);
-	g_print ("x_units: %lld\n", draw_context->renderer->priv->x_units_per_pixel);
-	g_print ("y_units: %lld\n", draw_context->renderer->priv->y_units_per_pixel);
-#endif
 }
 
 static void
