@@ -35,13 +35,12 @@
 #include <goffice/graph/gog-plot-impl.h>
 #include <goffice/graph/gog-renderer.h>
 #include <goffice/utils/go-format.h>
-#include <goffice/utils/format.h>
 #include <goffice/utils/go-math.h>
 #include <goffice/gtk/goffice-gtk.h>
 #include <goffice/gtk/go-format-sel.h>
 
 #include <gsf/gsf-impl-utils.h>
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 
 #include <gtk/gtkcellrenderertext.h>
 #include <gtk/gtkcelllayout.h>
@@ -220,11 +219,11 @@ map_discrete_calc_ticks (GogAxis *axis)
 	double maximum, minimum;
 	double tick_start, label_start;
 	int tick_nbr, label_nbr;
-	int i, j;
+	int i, j, index;
 	int major_tick, major_label;
 
-	major_tick = rint (gog_axis_get_entry (axis, GOG_AXIS_ELEM_MAJOR_TICK, NULL));
-	major_label = rint (gog_axis_get_entry (axis, GOG_AXIS_ELEM_MINOR_TICK, NULL));
+	major_tick = go_rint (gog_axis_get_entry (axis, GOG_AXIS_ELEM_MAJOR_TICK, NULL));
+	major_label = go_rint (gog_axis_get_entry (axis, GOG_AXIS_ELEM_MINOR_TICK, NULL));
 	if (major_tick < 1)
 		major_tick = 1;
 	if (major_label < 1)
@@ -256,16 +255,17 @@ map_discrete_calc_ticks (GogAxis *axis)
 		ticks[i].label = NULL;
 	}
 	for (i = 0, j = tick_nbr; i < label_nbr; i++, j++) {
-		ticks[j].position = label_start + (double) (i) * major_label;
+		ticks[j].position = go_rint (label_start + (double) (i) * major_label);
+		index = ticks[j].position - 1;
 		ticks[j].type = GOG_AXIS_TICK_NONE;
 		if (axis->labels != NULL) {
-			if (i * major_label < go_data_vector_get_len (axis->labels))
-				ticks[j].label = go_data_vector_get_str (axis->labels, i * major_label);
+			if (index < go_data_vector_get_len (axis->labels) && index >= 0)
+				ticks[j].label = go_data_vector_get_str (axis->labels, index);
 			else
 				ticks[j].label = NULL;
 		}
 		else
-			ticks[j].label = g_strdup_printf ("%d", i * major_label + 1);
+			ticks[j].label = g_strdup_printf ("%d", index + 1);
 	}
 
 	gog_axis_set_ticks (axis, tick_nbr + label_nbr, ticks);
@@ -449,7 +449,7 @@ map_linear_calc_ticks (GogAxis *axis)
 		if (fabs (ratio - rint (ratio)) < 1E-3) {
 			ticks[i].type = GOG_AXIS_TICK_MAJOR;
 				if (axis->assigned_format == NULL || 
-				    style_format_is_general (axis->assigned_format))
+				    go_format_is_general (axis->assigned_format))
 					ticks[i].label = go_format_value (axis->format, ticks[i].position);
 				else
 					ticks[i].label = go_format_value (axis->assigned_format, ticks[i].position);
@@ -631,7 +631,7 @@ map_log_calc_ticks (GogAxis *axis)
 			if (i % major_label == 0) {
 				ticks[count].type = GOG_AXIS_TICK_MAJOR;
 				if (axis->assigned_format == NULL || 
-				    style_format_is_general (axis->assigned_format))
+				    go_format_is_general (axis->assigned_format))
 					ticks[count].label = go_format_value (axis->format, ticks[count].position);
 				else
 					ticks[count].label = go_format_value (axis->assigned_format, ticks[count].position);
@@ -1440,7 +1440,7 @@ gog_axis_populate_editor (GogObject *gobj,
 	GogDataset *set = GOG_DATASET (gobj);
 	GladeXML *gui;
 
-	gui = go_libglade_new ("gog-axis-prefs.glade", "axis_pref_box", NULL, cc);
+	gui = go_libglade_new ("gog-axis-prefs.glade", "axis_pref_box", GETTEXT_PACKAGE, cc);
 	if (gui == NULL)
 		return;
 
@@ -1448,12 +1448,12 @@ gog_axis_populate_editor (GogObject *gobj,
 	table = GTK_TABLE (glade_xml_get_widget (gui, "bound_table"));
 	if (axis->is_discrete) {
 		static char const * const dim_names[] = {
-			NULL,
-			NULL,
+			N_("M_inimum"),
+			N_("M_aximum"),
 			N_("Categories between _ticks"),
 			N_("Categories between _labels")
 		};
-		for (i = GOG_AXIS_ELEM_MAJOR_TICK; i < GOG_AXIS_ELEM_CROSS_POINT ; i++)
+		for (i = GOG_AXIS_ELEM_MIN; i < GOG_AXIS_ELEM_CROSS_POINT ; i++)
 			make_dim_editor (set, table, i, dalloc, dim_names);
 	} else {
 		static char const * const dim_names[] = {
@@ -1502,7 +1502,7 @@ gog_axis_populate_editor (GogObject *gobj,
 	/* Format page */
 	if (!axis->is_discrete && gog_axis_get_atype (axis) != GOG_AXIS_PSEUDO_3D) {
 		w = go_format_sel_new ();
-		if (axis->assigned_format != NULL && !style_format_is_general (axis->assigned_format))
+		if (axis->assigned_format != NULL && !go_format_is_general (axis->assigned_format))
 			go_format_sel_set_style_format (GO_FORMAT_SEL (w),
 				axis->assigned_format);
 		else if (axis->format != NULL)
@@ -1855,7 +1855,7 @@ gog_axis_bound_changed (GogAxis *axis, GogObject *contrib)
 
 /* gog_axis_get_grid_line:
  * @axis: #GogAxis
- * @major: wether to retrieve major or minor grid line.
+ * @major: whether to retrieve major or minor grid line.
  *
  * Returns a pointer to GridLine object associated to given axis, NULL
  * if it doesn't exists.
@@ -1897,8 +1897,9 @@ gog_axis_view_padding_request (GogView *view,
 	GogAxis *axis = GOG_AXIS (view->model);
 	GogAxisType type = gog_axis_get_atype (axis);
 	GogObjectPosition pos;
+	GogAxisPosition axis_pos;
 	GogViewAllocation tmp = *bbox;
-	GogViewRequisition req;
+	GogViewRequisition req, available;
 	GogViewPadding label_padding, child_padding;
 	GSList *ptr;
 	double const pad_h = gog_renderer_pt2r_y (view->renderer, PAD_HACK);
@@ -1906,15 +1907,25 @@ gog_axis_view_padding_request (GogView *view,
 
 	label_padding.wr = label_padding.wl = label_padding.ht = label_padding.hb = 0;
 
+	axis_pos = GOG_AXIS_BASE (axis)->position;
+
 	for (ptr = view->children; ptr != NULL ; ptr = ptr->next) {
 		child = ptr->data;
 		pos = child->model->position;
 		if (IS_GOG_LABEL (child->model) && !(pos & GOG_POSITION_MANUAL)) {
-			gog_view_size_request (child, &req);
+			available.w = bbox->w;
+			available.h = bbox->h;
+			gog_view_size_request (child, &available, &req);
 			if (type == GOG_AXIS_X) 
-				label_padding.hb += req.h + pad_h;
+				if (axis_pos != GOG_AXIS_AT_HIGH)
+					label_padding.hb += req.h + pad_h;
+				else 
+					label_padding.ht += req.h + pad_h;
 			else  
-				label_padding.wl += req.w + pad_w;
+				if (axis_pos != GOG_AXIS_AT_HIGH)
+					label_padding.wl += req.w + pad_w;
+				else
+					label_padding.wr += req.w + pad_w;
 		}
 	}
 
@@ -1952,35 +1963,53 @@ gog_axis_view_size_allocate (GogView *view, GogViewAllocation const *bbox)
 	GogViewAllocation tmp = *bbox;
 	GogViewAllocation const *plot_area = gog_chart_view_get_plot_area (view->parent);
 	GogViewAllocation child_bbox;
-	GogViewRequisition req;
+	GogViewRequisition req, available;
 	GogObjectPosition pos;
+	GogAxisPosition axis_pos;
 	double const pad_h = gog_renderer_pt2r_y (view->renderer, PAD_HACK);
 	double const pad_w = gog_renderer_pt2r_x (view->renderer, PAD_HACK);
 
+	available.w = bbox->w;
+	available.h = bbox->h;
+
+	axis_pos = GOG_AXIS_BASE (axis)->position;
+
 	for (ptr = view->children; ptr != NULL ; ptr = ptr->next) {
 		child = ptr->data;
 		pos = child->model->position;
 		if (IS_GOG_LABEL (child->model) && (pos & GOG_POSITION_MANUAL)) {
-			gog_view_size_request (child, &req);
+			gog_view_size_request (child, &available, &req);
 			child_bbox = gog_object_get_manual_allocation (gog_view_get_model (child), 
 								       plot_area, &req);
 			gog_view_size_allocate (child, &child_bbox);
 		} else {
 			if (GOG_POSITION_IS_SPECIAL (pos)) {
 				if (IS_GOG_LABEL (child->model)) {
-					gog_view_size_request (child, &req);
+					gog_view_size_request (child, &available, &req);
 					if (type == GOG_AXIS_X) {
-						child_bbox.x = plot_area->x + (plot_area->w - req.w) / 2.0;
+						child_bbox.x = plot_area->x + 
+							(plot_area->w - req.w) / 2.0;
 						child_bbox.w = plot_area->w;
-						child_bbox.y = tmp.y + tmp.h - req.h;
 						child_bbox.h = req.h;
+						if (axis_pos != GOG_AXIS_AT_HIGH) {
+							child_bbox.y = tmp.y + tmp.h - req.h;
+						} else {
+							child_bbox.y = tmp.y;
+							tmp.y += req.h + pad_h;
+						}
 						tmp.h -= req.h + pad_h;
-					} else {
-						child_bbox.x = tmp.x;
-						child_bbox.w = req.w;
-						child_bbox.y = plot_area->y + (plot_area->h - req.h) / 2.0;
+					} else { 
+						child_bbox.y = plot_area->y + 
+							(plot_area->h - req.h) / 2.0;
 						child_bbox.h = plot_area->h;
-						tmp.x += req.w + pad_w;
+						child_bbox.w = req.w;
+						if (axis_pos != GOG_AXIS_AT_HIGH) {
+							child_bbox.x = tmp.x;
+							tmp.x += req.w + pad_w;
+						}
+						else {
+							child_bbox.x = tmp.x + tmp.w - req.w;
+						}
 						tmp.w -= req.w + pad_w;
 					}
 					gog_view_size_allocate (child, &child_bbox);
