@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  */
 
@@ -52,6 +52,8 @@ struct _GogRendererGnomePrint {
 	GPtrArray *fonts;
 	GnomePrintContext *gp_context;
 	PangoLayout *layout;
+
+	GogViewAllocation bbox;
 };
 
 typedef GogRendererClass GogRendererGnomePrintClass;
@@ -90,32 +92,6 @@ gog_renderer_gnome_print_finalize (GObject *obj)
 	(*parent_klass->finalize) (obj);
 }
 
-/*
- * print_make_rectangle_path
- * @pc      print context
- * @left    left side x coordinate
- * @bottom  bottom side y coordinate
- * @right   right side x coordinate
- * @top     top side y coordinate
- *
- * Make a rectangular path.
- * */
-
-static void
-print_make_rectangle_path (GnomePrintContext *pc,
-                           double left, double bottom,
-                           double right, double top)
-{
-        g_return_if_fail (pc != NULL);
-
-        gnome_print_newpath   (pc);
-        gnome_print_moveto    (pc, left, bottom);
-        gnome_print_lineto    (pc, left, top);
-        gnome_print_lineto    (pc, right, top);
-        gnome_print_lineto    (pc, right, bottom);
-        gnome_print_closepath (pc);
-}
-
 static PangoFontDescription *
 get_font (GogRendererGnomePrint *prend, GOFont const *gf)
 {
@@ -149,27 +125,6 @@ set_color (GogRendererGnomePrint *prend, GOColor color)
 	gnome_print_setopacity (prend->gp_context, a);
 }
 
-static void
-gog_renderer_gnome_print_clip_push (GogRenderer *rend, GogRendererClip *clip)
-{
-	GogRendererGnomePrint *prend = GOG_RENDERER_GNOME_PRINT (rend);
-
-	gnome_print_gsave (prend->gp_context);
-	print_make_rectangle_path (prend->gp_context,
-				   clip->area.x, -clip->area.y,
-				   clip->area.w + clip->area.x,
-				   -clip->area.h - clip->area.y);
-	gnome_print_clip (prend->gp_context);
-}
-
-static void
-gog_renderer_gnome_print_clip_pop (GogRenderer *rend, GogRendererClip *clip)
-{
-	GogRendererGnomePrint *prend = GOG_RENDERER_GNOME_PRINT (rend);
-
-	gnome_print_grestore (prend->gp_context);
-}
-
 static void
 draw_path (GogRendererGnomePrint *prend, ArtVpath const *path)
 {
@@ -191,16 +146,23 @@ draw_path (GogRendererGnomePrint *prend, ArtVpath const *path)
 }
 
 static void
-setup_clip (GogRendererGnomePrint *prend, GogViewAllocation const *bound)
+gog_renderer_gnome_print_push_clip (GogRenderer *rend, GogRendererClip *clip)
 {
+	GogRendererGnomePrint *prend = GOG_RENDERER_GNOME_PRINT (rend);
+
 	gnome_print_gsave (prend->gp_context);
-	gnome_print_moveto (prend->gp_context, bound->x, -bound->y);
-	gnome_print_lineto (prend->gp_context, bound->x + bound->w, -bound->y);
-	gnome_print_lineto (prend->gp_context, bound->x + bound->w, -bound->y - bound->h);
-	gnome_print_lineto (prend->gp_context, bound->x, -bound->y - bound->h);
+	draw_path (prend, clip->path);
 	gnome_print_clip (prend->gp_context);
 }
 
+static void
+gog_renderer_gnome_print_pop_clip (GogRenderer *rend, GogRendererClip *clip)
+{
+	GogRendererGnomePrint *prend = GOG_RENDERER_GNOME_PRINT (rend);
+
+	gnome_print_grestore (prend->gp_context);
+}
+
 static void
 set_dash (GogRendererGnomePrint *prend, ArtVpathDash *dash)
 {
@@ -212,8 +174,7 @@ set_dash (GogRendererGnomePrint *prend, ArtVpathDash *dash)
 }
 
 static void
-gog_renderer_gnome_print_draw_path (GogRenderer *renderer, ArtVpath const *path,
-				    GogViewAllocation const *bound)
+gog_renderer_gnome_print_draw_path (GogRenderer *renderer, ArtVpath const *path)
 {
 	GogRendererGnomePrint *prend = GOG_RENDERER_GNOME_PRINT (renderer);
 	GogStyle const *style = renderer->cur_style;
@@ -226,20 +187,14 @@ gog_renderer_gnome_print_draw_path (GogRenderer *renderer, ArtVpath const *path,
 	gnome_print_setlinewidth (prend->gp_context,
 		gog_renderer_line_size (renderer, style->line.width));
 
-	if (bound != NULL)
-		setup_clip (prend, bound);
-	
 	if (style->line.dash_type != GO_LINE_SOLID && renderer->cur_clip != NULL) {
-		ArtVpath *clipped = go_line_clip_vpath (path, &renderer->cur_clip->area);
+		ArtVpath *clipped = go_line_clip_vpath (path, &prend->bbox);
 		draw_path (prend, clipped);
 		g_free (clipped);
 	} else
 		draw_path (prend, path);
 
 	gnome_print_stroke (prend->gp_context);
-	
-	if (bound != NULL)
-		gnome_print_grestore (prend->gp_context);
 }
 
 static void
@@ -258,7 +213,7 @@ print_image (GogRendererGnomePrint *prend, GdkPixbuf *image, int w, int h)
 #define PIXBUF_SIZE 1024
 static void
 gog_renderer_gnome_print_draw_polygon (GogRenderer *renderer, ArtVpath const *path,
-				       gboolean narrow, GogViewAllocation const *bound)
+				       gboolean narrow)
 {
 	GogRendererGnomePrint *prend = GOG_RENDERER_GNOME_PRINT (renderer);
 	GogStyle const *style = renderer->cur_style;
@@ -271,14 +226,12 @@ gog_renderer_gnome_print_draw_polygon (GogRenderer *renderer, ArtVpath const *pa
 	ArtGradientLinear gradient;
 	ArtGradientStop stops[2];
 
-	if (bound != NULL)
-		setup_clip (prend, bound);
-
 	if (style->fill.type != GOG_FILL_STYLE_NONE || with_outline) {
 		if (style->outline.dash_type != GO_LINE_SOLID && renderer->cur_clip != NULL) {
-			ArtVpath *clipped = go_line_clip_vpath (path, &renderer->cur_clip->area);
+			ArtVpath *clipped = go_line_clip_vpath (path, &prend->bbox);
 			draw_path (prend, clipped);
 			g_free (clipped);
+			draw_path (prend, path);
 		} else
 			draw_path (prend, path);
 		gnome_print_closepath (prend->gp_context);
@@ -429,8 +382,6 @@ gog_renderer_gnome_print_draw_polygon (GogRenderer *renderer, ArtVpath const *pa
 			gog_renderer_line_size (renderer, style->outline.width));
 		gnome_print_stroke (prend->gp_context);
 	}
-	if (bound != NULL)
-		gnome_print_grestore (prend->gp_context);
 }
 
 static void
@@ -460,8 +411,7 @@ draw_bezier_path (GogRendererGnomePrint *prend, ArtBpath const *path)
 }
 
 static void
-gog_renderer_gnome_print_draw_bezier_path (GogRenderer *rend, ArtBpath const *path,
-			       GogViewAllocation const *bound)
+gog_renderer_gnome_print_draw_bezier_path (GogRenderer *rend, ArtBpath const *path)
 {
 	GogRendererGnomePrint *prend = GOG_RENDERER_GNOME_PRINT (rend);
 	GogStyle const *style = rend->cur_style;
@@ -474,29 +424,17 @@ gog_renderer_gnome_print_draw_bezier_path (GogRenderer *rend, ArtBpath const *pa
 	gnome_print_setlinewidth (prend->gp_context,
 		gog_renderer_line_size (rend, style->line.width));
 
-	if (bound != NULL)
-		setup_clip (prend, bound);
-	
 	draw_bezier_path (prend, path);
-/*	if (style->line.dash_type != GO_LINE_SOLID && renderer->cur_clip != NULL) {
-		ArtVpath *clipped = go_line_clip_vpath (path, &renderer->cur_clip->area);
-		draw_path (prend, clipped);
-		g_free (clipped);
-	} else
-		draw_path (prend, path);*/
 
 	gnome_print_stroke (prend->gp_context);
-	
-	if (bound != NULL)
-		gnome_print_grestore (prend->gp_context);
 }
 
 static void
 gog_renderer_gnome_print_draw_bezier_polygon (GogRenderer *rend, ArtBpath const *path,
-					      gboolean narrow, GogViewAllocation const *bound)
+					      gboolean narrow)
 {
 	ArtVpath *vpath = art_bez_path_to_vec (path, .1);
-	gog_renderer_gnome_print_draw_polygon (rend, vpath, narrow, bound);
+	gog_renderer_gnome_print_draw_polygon (rend, vpath, narrow);
 	art_free (vpath);
 }
 
@@ -509,56 +447,67 @@ gog_renderer_gnome_print_draw_text (GogRenderer *rend, char const *text,
 	PangoFontDescription *pango_font = get_font (prend,  rend->cur_style->font.font);
 
 	if (text[0]) {
-		double x, y, w, h;
+		GOColor fore_color = rend->cur_style->font.color;
+		GOGeometryOBR obr;
+		GOGeometryAABR aabr;
 		int iw, ih;
 
 		pango_layout_set_font_description (prend->layout, pango_font);
 		pango_layout_set_text (prend->layout, text, -1);
 		pango_layout_get_size (prend->layout, &iw, &ih);
-		w = iw / (double)PANGO_SCALE;
-		h = ih / (double)PANGO_SCALE;
-		x = pos->x;
-		switch (anchor) {
-		case GTK_ANCHOR_CENTER : case GTK_ANCHOR_N : case GTK_ANCHOR_S :
-			x -= w / 2.0;
-			break;
-		case GTK_ANCHOR_NE : case GTK_ANCHOR_SE : case GTK_ANCHOR_E :
-			x -= w;
-			break;
-		default : break;
-		}
-		if (x <= 0)
-			x = 0;
-	
-		y = pos->y;
-		switch (anchor) {
-		case GTK_ANCHOR_CENTER : case GTK_ANCHOR_E : case GTK_ANCHOR_W :
-			y -= h / 2.0;
-			break;
-		case GTK_ANCHOR_SE : case GTK_ANCHOR_S : case GTK_ANCHOR_SW :
-			y -= h;
-			break;
-		default : break;
-		}
-		if (y <= 0)
-			y = 0;
-		
-#warning "add clipping"
+		obr.w = iw / (double)PANGO_SCALE;
+		obr.h = ih / (double)PANGO_SCALE;
+		obr.alpha = rend->cur_style->text_layout.angle * M_PI / 180.0;
+		obr.x = pos->x;
+		obr.y = pos->y;
+		go_geometry_OBR_to_AABR (&obr, &aabr);
 
-		gnome_print_moveto (prend->gp_context,x, -y);
+		switch (anchor) {
+			case GTK_ANCHOR_NW: case GTK_ANCHOR_W: case GTK_ANCHOR_SW:
+				obr.x += aabr.w / 2.0;
+				break;
+			case GTK_ANCHOR_NE : case GTK_ANCHOR_SE : case GTK_ANCHOR_E :
+				obr.x -= aabr.w / 2.0;
+				break;
+			default : break;
+		}
+		if (obr.x <= 0)
+			obr.x = 0;
+
+		switch (anchor) {
+			case GTK_ANCHOR_NW: case GTK_ANCHOR_N: case GTK_ANCHOR_NE:
+				obr.y += aabr.h / 2.0;
+				break;
+			case GTK_ANCHOR_SE : case GTK_ANCHOR_S : case GTK_ANCHOR_SW :
+				obr.y -= aabr.h / 2.0;
+				break;
+			default : break;
+		}
+		if (obr.y <= 0)
+			obr.y = 0;
+
+		gnome_print_gsave (prend->gp_context);
+		gnome_print_setrgbcolor (prend->gp_context,
+			UINT_RGBA_R (fore_color) / 255.,
+			UINT_RGBA_G (fore_color) / 255.,
+			UINT_RGBA_B (fore_color) / 255.);
+		gnome_print_moveto (prend->gp_context, 
+				    obr.x - (obr.w / 2.0) * cos (obr.alpha) - (obr.h / 2.0) * sin (obr.alpha),
+				    -obr.y - (obr.w / 2.0) * sin (obr.alpha) + (obr.h / 2.0) * cos (obr.alpha));
+		gnome_print_rotate (prend->gp_context, rend->cur_style->text_layout.angle);
 		gnome_print_pango_layout (prend->gp_context, prend->layout);
+		gnome_print_grestore (prend->gp_context);
 		if (result != NULL) {
-			result->x = x;
-			result->y = y;
-			result->w = w;
-			result->h = h;
+			result->x = aabr.x;
+			result->y = aabr.y;
+			result->w = aabr.w;
+			result->h = aabr.h;
 		}
 	}
 }
 
 static void
-gog_renderer_gnome_print_measure_text (GogRenderer *rend,
-				       char const *text, GogViewRequisition *size)
+gog_renderer_gnome_print_get_text_OBR (GogRenderer *rend, char const *text, GOGeometryOBR *obr)
 {
 	GogRendererGnomePrint *prend = GOG_RENDERER_GNOME_PRINT (rend);
 	PangoFontDescription *pango_font = get_font (prend,  rend->cur_style->font.font);
@@ -567,8 +516,8 @@ gog_renderer_gnome_print_measure_text (GogRenderer *rend,
 	pango_layout_set_font_description (prend->layout, pango_font);
 	pango_layout_set_text (prend->layout, text, -1);
 	pango_layout_get_size (prend->layout, &iw, &ih);
-	size->w = iw / (double)PANGO_SCALE;
-	size->h = ih / (double)PANGO_SCALE;
+	obr->w = iw / (double)PANGO_SCALE;
+	obr->h = ih / (double)PANGO_SCALE;
 }
 
 static void
@@ -591,7 +540,7 @@ gog_renderer_gnome_print_draw_marker (GogRenderer *renderer, double x, double y)
 
 	gnome_print_gsave (prend->gp_context);
 
-	half_size = gog_renderer_line_size (renderer, marker->size) / 2.0;
+	half_size = gog_renderer_line_size (renderer, go_marker_get_size (marker)) / 2.0;
 	art_affine_scale (scaling, half_size, half_size);
 	art_affine_translate (translation, x, y);
 	art_affine_multiply (affine, scaling, translation);
@@ -600,12 +549,12 @@ gog_renderer_gnome_print_draw_marker (GogRenderer *renderer, double x, double y)
 	fill_path = art_vpath_affine_transform (fill_path_raw, affine);
 
 	gnome_print_setlinecap (prend->gp_context, ART_PATH_STROKE_CAP_ROUND);
-	set_color (prend, marker->fill_color);
+	set_color (prend, go_marker_get_fill_color (marker));
 	draw_path (prend, fill_path);
 	gnome_print_closepath (prend->gp_context);
 	gnome_print_fill (prend->gp_context);
 
-	set_color (prend, marker->outline_color);
+	set_color (prend, go_marker_get_outline_color (marker));
 	gnome_print_setlinewidth (prend->gp_context,
 		gog_renderer_line_size (renderer,
 					go_marker_get_outline_width (marker)));
@@ -626,15 +575,15 @@ gog_renderer_gnome_print_class_init (GogRendererClass *rend_klass)
 
 	parent_klass = g_type_class_peek_parent (rend_klass);
 	gobject_klass->finalize	  	= gog_renderer_gnome_print_finalize;
-	rend_klass->clip_push  		= gog_renderer_gnome_print_clip_push;
-	rend_klass->clip_pop		= gog_renderer_gnome_print_clip_pop;
+	rend_klass->push_clip  		= gog_renderer_gnome_print_push_clip;
+	rend_klass->pop_clip		= gog_renderer_gnome_print_pop_clip;
 	rend_klass->draw_path	  	= gog_renderer_gnome_print_draw_path;
 	rend_klass->draw_polygon  	= gog_renderer_gnome_print_draw_polygon;
 	rend_klass->draw_bezier_path 	= gog_renderer_gnome_print_draw_bezier_path;
 	rend_klass->draw_bezier_polygon = gog_renderer_gnome_print_draw_bezier_polygon;
 	rend_klass->draw_text	  	= gog_renderer_gnome_print_draw_text;
 	rend_klass->draw_marker	  	= gog_renderer_gnome_print_draw_marker;
-	rend_klass->measure_text  	= gog_renderer_gnome_print_measure_text;
+	rend_klass->get_text_OBR	= gog_renderer_gnome_print_get_text_OBR;
 }
 
 static void
@@ -642,6 +591,10 @@ gog_renderer_gnome_print_init (GogRendererGnomePrint *prend)
 {
 	prend->gp_context = NULL;
 	prend->fonts = g_ptr_array_new ();
+	prend->bbox.x =
+	prend->bbox.y = 0.0;
+	prend->bbox.w =
+	prend->bbox.h = 1.0;
 }
 
 static GSF_CLASS (GogRendererGnomePrint, gog_renderer_gnome_print,
@@ -669,6 +622,7 @@ gog_graph_print_to_gnome_print (GogGraph *graph,
 	allocation.w = width;
 	allocation.h = height;
 	gog_view_size_allocate (prend->base.view, &allocation);
+	prend->bbox = allocation;
 	
 	/* FIXME FIXME FIXME this is a workaround for a bug in libgnomeprint
 	 * where line with width == 1.0 don't scale properly before an other
