@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  */
 
@@ -83,54 +83,78 @@ gog_renderer_pixbuf_finalize (GObject *obj)
 
 typedef struct
 {
-	GdkPixbuf	*buffer;
-	double		 x_offset;
-	double		 y_offset;
+	GdkPixbuf *buffer;
+	double	   x_offset;
+	double	   y_offset;
+	ArtSVP	  *svp;	
 } ClipData;
 
 static void
-gog_renderer_pixbuf_clip_push (GogRenderer *rend, GogRendererClip *clip)
+gog_renderer_pixbuf_push_clip (GogRenderer *rend, GogRendererClip *clip)
 {
-	ClipData *clip_data;
-	GdkRectangle graph_rect, clip_rect, res_rect;
 	GogRendererPixbuf *prend = GOG_RENDERER_PIXBUF (rend);
+	ClipData *clip_data;
+	ArtVpath *path = clip->path;
+	int i;
+	gboolean is_rectangle;
+
+	for (i = 0; i < 6; i++)
+		if (path[i].code == ART_END)
+			break;
+	
+	is_rectangle = i == 5 &&
+		path[5].code == ART_END &&
+		path[0].x == path[3].x &&
+		path[0].x == path[4].x &&
+		path[1].x == path[2].x &&
+		path[0].y == path[1].y &&
+	       	path[0].y == path[4].y &&
+		path[2].y == path[3].y;
 
 	clip->data = g_new (ClipData, 1);
 	clip_data = (ClipData *) clip->data;
-
 	clip_data->x_offset = prend->x_offset;
 	clip_data->y_offset = prend->y_offset;
 	clip_data->buffer = NULL;
+	clip_data->svp = NULL;
 
-	graph_rect.x = graph_rect.y = 0;
-	graph_rect.width = gdk_pixbuf_get_width (prend->buffer);
-	graph_rect.height = gdk_pixbuf_get_height (prend->buffer);
+	if (is_rectangle) {
+		GdkRectangle graph_rect, clip_rect, res_rect;
 
-	clip_rect.x = floor (clip->area.x - prend->x_offset + 0.5);
-	clip_rect.y = floor (clip->area.y - prend->y_offset + 0.5);
-	clip_rect.width = floor (clip->area.x - prend->x_offset + clip->area.w + 0.5) - clip_rect.x;
-	clip_rect.height = floor (clip->area.y -prend->y_offset + clip->area.h + 0.5) - clip_rect.y;
+		graph_rect.x = graph_rect.y = 0;
+		graph_rect.width = gdk_pixbuf_get_width (prend->buffer);
+		graph_rect.height = gdk_pixbuf_get_height (prend->buffer);
 
-	if (gdk_rectangle_intersect (&graph_rect, &clip_rect, &res_rect)) {
-		clip_data->buffer = prend->buffer;
-		prend->buffer = gdk_pixbuf_new_subpixbuf (clip_data->buffer,
-							  res_rect.x, res_rect.y,
-							  res_rect.width, res_rect.height);
-		prend->x_offset += res_rect.x;
-		prend->y_offset += res_rect.y;
+		clip_rect.x = floor (path[0].x - prend->x_offset + 0.5);
+		clip_rect.y = floor (path[0].y - prend->y_offset + 0.5);
+		clip_rect.width = floor (path[1].x - prend->x_offset + 0.5) - clip_rect.x;
+		clip_rect.height = floor (path[2].y -prend->y_offset + 0.5) - clip_rect.y;
+
+		if (gdk_rectangle_intersect (&graph_rect, &clip_rect, &res_rect)) {
+			clip_data->buffer = prend->buffer;
+			prend->buffer = gdk_pixbuf_new_subpixbuf (clip_data->buffer,
+								  res_rect.x, res_rect.y,
+								  res_rect.width, res_rect.height);
+			prend->x_offset += res_rect.x;
+			prend->y_offset += res_rect.y;
+		}
+
+		if (prend->buffer == NULL)
+			g_warning ("Pixbuf renderer: invalid clipping region");
+
+		prend->pixels = gdk_pixbuf_get_pixels (prend->buffer);
+		prend->w = gdk_pixbuf_get_width (prend->buffer);
+		prend->h = gdk_pixbuf_get_height (prend->buffer);
+		prend->rowstride = gdk_pixbuf_get_rowstride (prend->buffer);
+	} else {
+		/* We cheat and assume new clip area is included
+		 * in previous one */
+		clip_data->svp = art_svp_from_vpath (path);
 	}
-
-	if (prend->buffer == NULL)
-		g_warning ("Pixbuf renderer: invalid clipping region");
-
-	prend->pixels = gdk_pixbuf_get_pixels (prend->buffer);
-	prend->w = gdk_pixbuf_get_width (prend->buffer);
-	prend->h = gdk_pixbuf_get_height (prend->buffer);
-	prend->rowstride = gdk_pixbuf_get_rowstride (prend->buffer);
 }
 
 static void
-gog_renderer_pixbuf_clip_pop (GogRenderer *rend, GogRendererClip *clip)
+gog_renderer_pixbuf_pop_clip (GogRenderer *rend, GogRendererClip *clip)
 {
 	GogRendererPixbuf *prend = GOG_RENDERER_PIXBUF (rend);
 	ClipData *clip_data = clip->data;
@@ -139,35 +163,42 @@ gog_renderer_pixbuf_clip_pop (GogRenderer *rend, GogRendererClip *clip)
 		if (prend->buffer != NULL)
 			g_object_unref (prend->buffer);
 		prend->buffer = clip_data->buffer;
+		prend->pixels = gdk_pixbuf_get_pixels (prend->buffer);
+		prend->w = gdk_pixbuf_get_width (prend->buffer);
+		prend->h = gdk_pixbuf_get_height (prend->buffer);
+		prend->rowstride = gdk_pixbuf_get_rowstride (prend->buffer);
+		prend->x_offset = clip_data->x_offset;
+		prend->y_offset = clip_data->y_offset;
 	}
-	prend->pixels = gdk_pixbuf_get_pixels (prend->buffer);
-	prend->w = gdk_pixbuf_get_width (prend->buffer);
-	prend->h = gdk_pixbuf_get_height (prend->buffer);
-	prend->rowstride = gdk_pixbuf_get_rowstride (prend->buffer);
-	prend->x_offset = clip_data->x_offset;
-	prend->y_offset = clip_data->y_offset;
+	if (clip_data->svp != NULL)
+		art_free (clip_data->svp);
 
 	g_free (clip->data);
 	clip->data = NULL;
 }
 
-static ArtSVP *
-clip_path (GogViewAllocation const *bound)
+static void
+gog_renderer_pixbuf_do_clip (GogRenderer *rend, ArtSVP **svp)
 {
-	ArtVpath path[6];
-	path[0].code = ART_MOVETO;
-	path[1].code = ART_LINETO;
-	path[2].code = ART_LINETO;
-	path[3].code = ART_LINETO;
-	path[4].code = ART_LINETO;
-	path[5].code = ART_END;
-	path[0].x = path[1].x = path[4].x = bound->x;
-	path[2].x = path[3].x = path[0].x + bound->w;
-	path[0].y = path[3].y = path[4].y = bound->y;
-	path[1].y = path[2].y = path[0].y + bound->h;
-	return art_svp_from_vpath ((ArtVpath *)path);
+	ClipData *clip_data;
+	ArtSVP *svp1;
+
+	g_return_if_fail (*svp != NULL);
+	
+       	if (rend->cur_clip == NULL)
+		return;
+
+	clip_data = rend->cur_clip->data;
+
+	if (clip_data->svp == NULL)
+		return;
+
+	svp1 = art_svp_intersect (*svp, clip_data->svp);
+	art_free (*svp);
+	*svp = svp1;
 }
 
+
 static double
 line_size (GogRenderer const *rend, double width)
 {
@@ -212,10 +243,10 @@ gog_renderer_pixbuf_sharp_path (GogRenderer *rend, ArtVpath *path, double line_w
 }
 
 static void
-gog_renderer_pixbuf_draw_path (GogRenderer *rend, ArtVpath const *path,
-			       GogViewAllocation const *bound)
+gog_renderer_pixbuf_draw_path (GogRenderer *rend, ArtVpath const *path)
 {
 	GogRendererPixbuf *prend = GOG_RENDERER_PIXBUF (rend);
+	GogViewAllocation area;
 	GogStyle const *style = rend->cur_style;
 	double width = line_size (rend, style->line.width);
 	ArtSVP *svp;
@@ -244,14 +275,7 @@ gog_renderer_pixbuf_draw_path (GogRenderer *rend, ArtVpath const *path,
 									ART_PATH_STROKE_JOIN_MITER, 
 									ART_PATH_STROKE_CAP_ROUND,
 									width, 4, 0.5);
-					if (bound != NULL) {
-						ArtSVP *orig = svp;
-						ArtSVP *clip = clip_path (bound);
-						svp = art_svp_intersect (clip, orig);
-						art_svp_free (clip);
-						art_svp_free (orig);
-					}
-				
+					gog_renderer_pixbuf_do_clip (rend, &svp);
 					go_color_render_svp (style->line.color, svp,
 								 prend->x_offset,
 								 prend->y_offset,
@@ -276,8 +300,11 @@ gog_renderer_pixbuf_draw_path (GogRenderer *rend, ArtVpath const *path,
 					short_path[1].y = path [i].y;
 					dx = short_path[1].x - short_path[0].x;
 					dy = short_path[1].y - short_path[0].y;
-					dashed_path = go_line_dash_vpath (short_path, rend->line_dash,  
-						rend->cur_clip != NULL ? &rend->cur_clip->area : NULL);
+					area.x = prend->x_offset;
+					area.y = prend->y_offset;
+					area.w = prend->w;
+					area.h = prend->h;
+					dashed_path = go_line_dash_vpath (short_path, rend->line_dash, &area);  
 					dx = sqrt (dx * dx + dy * dy);
 					n = floor (dx / dash_length);
 					rend->line_dash->offset += dx - n * dash_length;
@@ -287,21 +314,15 @@ gog_renderer_pixbuf_draw_path (GogRenderer *rend, ArtVpath const *path,
 										ART_PATH_STROKE_CAP_ROUND,
 										width, 4, 0.5);
 						g_free (dashed_path);
-					if (bound != NULL) {
-						ArtSVP *orig = svp;
-						ArtSVP *clip = clip_path (bound);
-						svp = art_svp_intersect (clip, orig);
-						art_svp_free (clip);
-						art_svp_free (orig);
-					}
-				
-					go_color_render_svp (style->line.color, svp,
-								 prend->x_offset,
-								 prend->y_offset,
-								 prend->w + prend->x_offset,
-								 prend->h + prend->y_offset,
-								 prend->pixels, prend->rowstride);
-					art_svp_free (svp);
+						
+						gog_renderer_pixbuf_do_clip (rend, &svp);
+						go_color_render_svp (style->line.color, svp,
+								     prend->x_offset,
+								     prend->y_offset,
+								     prend->w + prend->x_offset,
+								     prend->h + prend->y_offset,
+								     prend->pixels, prend->rowstride);
+						art_svp_free (svp);
 					}
 				}
 				i++;
@@ -311,11 +332,10 @@ gog_renderer_pixbuf_draw_path (GogRenderer *rend, ArtVpath const *path,
 }
   
 static void
-gog_renderer_pixbuf_draw_bezier_path (GogRenderer *rend, ArtBpath const *path,
-				      GogViewAllocation const *bound)
+gog_renderer_pixbuf_draw_bezier_path (GogRenderer *rend, ArtBpath const *path)
 {
 	ArtVpath *vpath = art_bez_path_to_vec (path, .1);
-	gog_renderer_pixbuf_draw_path (rend, vpath, bound);
+	gog_renderer_pixbuf_draw_path (rend, vpath);
 	art_free (vpath);
 }
 
@@ -332,11 +352,11 @@ gog_art_renderer_new (GogRendererPixbuf *prend)
 }
 
 static void
-gog_renderer_pixbuf_draw_polygon (GogRenderer *rend, ArtVpath const *path,
-				  gboolean narrow, GogViewAllocation const *bound)
+gog_renderer_pixbuf_draw_polygon (GogRenderer *rend, ArtVpath const *path, gboolean narrow)
 {
 	GogRendererPixbuf *prend = GOG_RENDERER_PIXBUF (rend);
 	GogStyle const *style = rend->cur_style;
+	GogViewAllocation area;
 	ArtVpath *dashed_path;
 	ArtRender *render;
 	ArtSVP *outline = NULL;
@@ -358,8 +378,11 @@ gog_renderer_pixbuf_draw_polygon (GogRenderer *rend, ArtVpath const *path,
 								width, 4, 0.5);
 				break;
 			default:
-				dashed_path = go_line_dash_vpath (path, rend->outline_dash,  
-					rend->cur_clip != NULL ? &rend->cur_clip->area : NULL);
+				area.x = prend->x_offset;
+				area.y = prend->y_offset;
+				area.w = prend->w;
+				area.h = prend->h;
+				dashed_path = go_line_dash_vpath (path, rend->outline_dash, &area);  
 				if (dashed_path != NULL) {
 					outline = art_svp_vpath_stroke (dashed_path,
 									ART_PATH_STROKE_JOIN_MITER, 
@@ -368,13 +391,6 @@ gog_renderer_pixbuf_draw_polygon (GogRenderer *rend, ArtVpath const *path,
 					g_free (dashed_path);
 				}
 		}
-		if (bound != NULL && outline != NULL) {
-			ArtSVP *orig = outline;
-			ArtSVP *clip = clip_path (bound);
-			outline = art_svp_intersect (clip, orig);
-			art_svp_free (clip);
-			art_svp_free (orig);
-		}
 	}
 
 	if (style->fill.type != GOG_FILL_STYLE_NONE) {
@@ -387,20 +403,7 @@ gog_renderer_pixbuf_draw_polygon (GogRenderer *rend, ArtVpath const *path,
 		fill = art_svp_rewind_uncrossed (svp2, ART_WIND_RULE_NONZERO);
 		art_svp_free (svp1);
 		art_svp_free (svp2);
-		if (bound != NULL) {
-			ArtSVP *orig = fill;
-			ArtSVP *clip = clip_path (bound);
-			fill = art_svp_intersect (clip, orig);
-			art_svp_free (clip);
-			art_svp_free (orig);
-		}
-#if 0 /* art_svp_minus is not implemented */
-		if (outline != NULL) {
-			ArtSVP *tmp = art_svp_minus (fill, outline);
-			art_svp_free (fill);
-			fill = tmp;
-		}
-#endif
+		gog_renderer_pixbuf_do_clip (rend, &fill);
 
 		switch (style->fill.type) {
 		case GOG_FILL_STYLE_PATTERN:
@@ -494,13 +497,13 @@ gog_renderer_pixbuf_draw_polygon (GogRenderer *rend, ArtVpath const *path,
 											     &dest_rect,
 											     &copy_rect))
 									gdk_pixbuf_copy_area (image,
-											      copy_rect.x - image_rect.x,
-											      copy_rect.y - image_rect.y,
-											      copy_rect.width,
-											      copy_rect.height,
-											      prend->buffer,
-											      copy_rect.x,
-											      copy_rect.y);
+										copy_rect.x - image_rect.x,
+										copy_rect.y - image_rect.y,
+										copy_rect.width,
+										copy_rect.height,
+										prend->buffer,
+										copy_rect.x,
+										copy_rect.y);
 								image_rect.y += image_rect.height;
 							}
 							image_rect.x +=image_rect.width;
@@ -521,6 +524,7 @@ gog_renderer_pixbuf_draw_polygon (GogRenderer *rend, ArtVpath const *path,
 	}
 
 	if (outline != NULL) {
+		gog_renderer_pixbuf_do_clip (rend, &outline);
 		go_color_render_svp (style->outline.color, outline,
 				     prend->x_offset,
 				     prend->y_offset,
@@ -533,13 +537,60 @@ gog_renderer_pixbuf_draw_polygon (GogRenderer *rend, ArtVpath const *path,
 
 static void
 gog_renderer_pixbuf_draw_bezier_polygon (GogRenderer *rend, ArtBpath const *path,
-					 gboolean narrow, GogViewAllocation const *bound)
+					 gboolean narrow)
 {
 	ArtVpath *vpath = art_bez_path_to_vec (path, .1);
-	gog_renderer_pixbuf_draw_polygon (rend, vpath, narrow, bound);
+	gog_renderer_pixbuf_draw_polygon (rend, vpath, narrow);
 	art_free (vpath);
 }
 
+static void
+get_rotated_layout_bounds (PangoLayout  *layout,
+			   PangoRectangle *rect)
+{
+	PangoContext *context = pango_layout_get_context (layout);
+	const PangoMatrix *matrix = pango_context_get_matrix (context);
+	gdouble x_min = 0, x_max = 0, y_min = 0, y_max = 0; /* quiet gcc */
+	PangoRectangle logical_rect;
+	gint i, j;
+
+	pango_layout_get_extents (layout, NULL, &logical_rect);
+
+	for (i = 0; i < 2; i++)
+	{
+		gdouble x = (i == 0) ? logical_rect.x : logical_rect.x + logical_rect.width;
+		for (j = 0; j < 2; j++)
+		{
+			gdouble y = (j == 0) ? logical_rect.y : logical_rect.y + logical_rect.height;
+
+			gdouble xt = (x * matrix->xx + y * matrix->xy) / PANGO_SCALE + matrix->x0;
+			gdouble yt = (x * matrix->yx + y * matrix->yy) / PANGO_SCALE + matrix->y0;
+
+			if (i == 0 && j == 0)
+			{
+				x_min = x_max = xt;
+				y_min = y_max = yt;
+			}
+			else
+			{
+				if (xt < x_min)
+					x_min = xt;
+				if (yt < y_min)
+					y_min = yt;
+				if (xt > x_max)
+					x_max = xt;
+				if (yt > y_max)
+					y_max = yt;
+			}
+		}
+	}
+
+	rect->x = floor (x_min);
+	rect->width = ceil (x_max) - rect->x;
+	rect->y = floor (y_min);
+	rect->height = floor (y_max) - rect->y;
+}
+
 static PangoContext *
 gog_renderer_pixbuf_get_pango_context (GogRendererPixbuf *prend)
 {
@@ -549,8 +600,8 @@ gog_renderer_pixbuf_get_pango_context (GogRendererPixbuf *prend)
 		return prend->pango_context;
 
 	font_map = PANGO_FT2_FONT_MAP (pango_ft2_font_map_new ());
-	pango_ft2_font_map_set_resolution (font_map,
-		prend->dpi_x, prend->dpi_y);
+	pango_ft2_font_map_set_resolution (font_map, prend->dpi_x, prend->dpi_y);
+
 	prend->pango_context = pango_ft2_font_map_create_context (font_map);
 	/*  Workaround for bug #143542 (PangoFT2Fontmap leak),
 	 *  see also bug #148997 (Text layer rendering leaks font file descriptor):
@@ -570,15 +621,26 @@ gog_renderer_pixbuf_get_pango_context (GogRendererPixbuf *prend)
 static PangoLayout *
 gog_renderer_pixbuf_get_pango_layout (GogRendererPixbuf *prend)
 {
+	GogRenderer *rend = GOG_RENDERER (prend);
+	GogStyle const *style = prend->base.cur_style;
 	PangoContext *context;
 	PangoAttribute *attr;
 	PangoAttrList  *attrs = NULL;
-	PangoFontDescription const *fd = prend->base.cur_style->font.font->desc;
+	PangoFontDescription const *fd = style->font.font->desc;
+	PangoMatrix matrix = PANGO_MATRIX_INIT;
 
 	if (prend->pango_layout != NULL)
 		return prend->pango_layout;
 	
 	context = gog_renderer_pixbuf_get_pango_context (prend);
+	switch (go_geometry_get_rotation_type (rend->cur_style->text_layout.angle * M_PI / 180.0)) {
+		case GO_ROTATE_FREE:
+			pango_matrix_rotate (&matrix, rend->cur_style->text_layout.angle);
+			pango_context_set_matrix (context, &matrix);
+			break;
+		default:
+			pango_context_set_matrix (context, NULL);
+	}	
 
 	prend->pango_layout = pango_layout_new (context);
 	
@@ -610,32 +672,75 @@ gog_renderer_pixbuf_draw_text (GogRenderer *rend, char const *text,
 	FT_Bitmap ft_bitmap;
 	GogRendererPixbuf *prend = GOG_RENDERER_PIXBUF (rend);
 	PangoRectangle rect;
-	PangoLayout   *layout;
+	PangoLayout *layout;
+	double angle;
 	guint8 r, g, b, a, alpha, *dst, *src;
-	int h, w, i, x, y;
+	int h, w, i, x, y, interrow, intercol, rotation_type;
+	int ft_x, ft_y, ft_w, ft_h, offset_x, offset_y;
+	int end_x = 0, end_y = 0;
 	GogStyle const *style = rend->cur_style;
+	PangoMatrix matrix, old_matrix;
+
+	angle = rend->cur_style->text_layout.angle * M_PI / 180.0;
+	rotation_type  = go_geometry_get_rotation_type (angle);
 
 	layout = gog_renderer_pixbuf_get_pango_layout ((GogRendererPixbuf *) rend);
 	pango_layout_set_text (layout, text, -1);
-	pango_layout_get_extents (layout, NULL, &rect);
-	rect.x = PANGO_PIXELS (rect.x);
-	rect.y = PANGO_PIXELS (rect.y);
+
+	switch (rotation_type) {
+		case GO_ROTATE_FREE:
+			get_rotated_layout_bounds (layout, &rect);
+			matrix = *pango_context_get_matrix (pango_layout_get_context (layout));
+			old_matrix = matrix;
+			matrix.x0 = -rect.x;
+			matrix.y0 = -rect.y;
+			pango_context_set_matrix (pango_layout_get_context (layout), &matrix);
+			rect.width *= PANGO_SCALE;
+			rect.height *= PANGO_SCALE;
+			rect.x = rect.y = 0;
+			break;
+		default:
+			pango_layout_get_extents (layout, NULL, &rect);
+			rect.x = PANGO_PIXELS (rect.x);
+			rect.y = PANGO_PIXELS (rect.y);
+	}
+
+	switch (rotation_type) {
+		case GO_ROTATE_CLOCKWISE:
+		case GO_ROTATE_COUNTERCLOCKWISE:
+			x = rect.x;
+			rect.x = rect.y;
+			rect.y = x;
+			w = rect.width;
+			rect.width = rect.height;
+			rect.height = w;
+			break;
+		default:
+			break;
+	}		
+
 	x = (int)((pos->x - prend->x_offset) * PANGO_SCALE);
 	y = (int)((pos->y - prend->y_offset) * PANGO_SCALE);
-	switch (anchor) {
-	case GTK_ANCHOR_CENTER : case GTK_ANCHOR_N : case GTK_ANCHOR_S :
-		x -= rect.width / 2;
-		break;
-	case GTK_ANCHOR_NE : case GTK_ANCHOR_SE : case GTK_ANCHOR_E :
-		x -= rect.width;
-		break;
-	default : break;
-	}
-	x = (x > 0) ? (x + PANGO_SCALE / 2) / PANGO_SCALE : 0;
-	w = (rect.width + PANGO_SCALE / 2) / PANGO_SCALE;
-	if ((x + w) > prend->w)
-		w = prend->w - x;
 
+	switch (anchor) {
+		case GTK_ANCHOR_CENTER : case GTK_ANCHOR_N : case GTK_ANCHOR_S :
+			x -= rect.width / 2;
+			break;
+		case GTK_ANCHOR_NE : case GTK_ANCHOR_SE : case GTK_ANCHOR_E :
+			x -= rect.width;
+			break;
+		default : break;
+	}
+	x = (x + PANGO_SCALE / 2) / PANGO_SCALE;
+	w = (rect.width + PANGO_SCALE / 2) / PANGO_SCALE;
+	offset_x = -MIN (x, 0);
+	x = MAX (x, 0);
+	w -= offset_x;
+	if (w + x - offset_x > prend->w) {
+		end_x = w - (prend->w - x);
+		w = prend->w - x;	
+	}
+	
 	switch (anchor) {
 	case GTK_ANCHOR_CENTER : case GTK_ANCHOR_E : case GTK_ANCHOR_W :
 		y -= rect.height / 2;
@@ -645,10 +750,15 @@ gog_renderer_pixbuf_draw_text (GogRenderer *rend, char const *text,
 		break;
 	default : break;
 	}
-	y = (y > 0) ? (y + PANGO_SCALE / 2) / PANGO_SCALE : 0;
+	y = (y + PANGO_SCALE / 2) / PANGO_SCALE;
 	h = (rect.height + PANGO_SCALE / 2) / PANGO_SCALE;
-	if ((y + h) > prend->h)
+	offset_y = -MIN (y, 0);
+	y = MAX (y, 0);
+	h -= offset_y;
+	if (h + y > prend->h) {
+		end_y = h - (prend->h - y);
 		h = prend->h - y;
+	}
 
 	if (result != NULL) {
 		result->x = x;
@@ -657,18 +767,53 @@ gog_renderer_pixbuf_draw_text (GogRenderer *rend, char const *text,
 		result->h = h;
 	}
 
-	if (w <= 0 || h <= 0) 
+	if (w <= 0 || h <= 0) { 
+		if (rotation_type == GO_ROTATE_FREE) 
+			pango_context_set_matrix (pango_layout_get_context (layout), &old_matrix);
 		return;
+	}
 
-	ft_bitmap.rows         = h;
-	ft_bitmap.width        = w;
-	ft_bitmap.pitch        = (w+3) & ~3;
+	switch (rotation_type) {
+		case GO_ROTATE_CLOCKWISE:
+			ft_h = w;
+			ft_w = h;
+			ft_x = offset_y;
+			ft_y = end_x;
+			break;
+		case GO_ROTATE_COUNTERCLOCKWISE:
+			ft_h = w;
+			ft_w = h;
+			ft_x = end_y;
+			ft_y = offset_x;
+			break;
+		case GO_ROTATE_UPSIDEDOWN:
+			ft_h = h;
+			ft_w = w;
+			ft_x = end_x;
+			ft_y = end_y;
+			break;
+		case GO_ROTATE_FREE:
+			ft_h = h;
+			ft_w = w;
+			ft_x = offset_x * cos (angle) + offset_y * sin (-angle);
+			ft_y = offset_x * sin (angle) + offset_y * cos (-angle);
+			break;
+		default:
+			ft_h = h;
+			ft_w = w;
+			ft_x = offset_x;
+			ft_y = offset_y;
+	}
+
+	ft_bitmap.rows         = ft_h;
+	ft_bitmap.width        = ft_w;
+	ft_bitmap.pitch        = (ft_w + 3) & ~3;
 	ft_bitmap.buffer       = g_malloc0 (ft_bitmap.rows * ft_bitmap.pitch);
 	ft_bitmap.num_grays    = 256;
 	ft_bitmap.pixel_mode   = ft_pixel_mode_grays;
 	ft_bitmap.palette_mode = 0;
 	ft_bitmap.palette      = NULL;
-	pango_ft2_render_layout (&ft_bitmap, layout, -rect.x, -rect.y);
+	pango_ft2_render_layout (&ft_bitmap, layout, -ft_x, -ft_y);
 
 	r = UINT_RGBA_R (style->font.color);
 	g = UINT_RGBA_G (style->font.color);
@@ -679,12 +824,40 @@ gog_renderer_pixbuf_draw_text (GogRenderer *rend, char const *text,
 	 * slow, and I do not feel like leaping through 20 different data
 	 * structures to composite 1 byte images, onto rgba */
 	dst = prend->pixels;
-	dst += (y * prend->rowstride);
-	dst += (x + rect.x)* 4;
 	src = ft_bitmap.buffer;
 
-	while (h--) {
-		for (i = w; i-- > 0 ; dst += 4, src++) {
+	switch (go_geometry_get_rotation_type (rend->cur_style->text_layout.angle * M_PI / 180.0)) {
+		case GO_ROTATE_FREE:
+		case GO_ROTATE_NONE:
+			intercol = 4;
+			interrow = prend->rowstride - w * 4;
+			dst += y * prend->rowstride;
+			dst += x * 4;
+			break;
+		case GO_ROTATE_COUNTERCLOCKWISE:
+			intercol = - prend->rowstride;
+			interrow = 4 + h * prend->rowstride; 
+			dst += (y + h - 1) * prend->rowstride;
+			dst += x * 4;
+			break;
+		case GO_ROTATE_CLOCKWISE:
+			intercol = prend->rowstride;
+			interrow = - 4 - h * prend->rowstride; 
+			dst += y * prend->rowstride;
+			dst += (x + w - 1)* 4;
+			break;
+		case GO_ROTATE_UPSIDEDOWN:
+			intercol = -4;
+			interrow = - prend->rowstride + w * 4; 
+			dst += (y + h - 1) * prend->rowstride;
+			dst += (x + w - 1)* 4;
+			break;
+		default:
+			intercol = interrow = 0;  /* never reached; silence gcc */
+	}
+
+	while (ft_h--) {
+		for (i = ft_w; i-- > 0 ; dst += intercol, src++) {
 			/* FIXME: Do the libart thing instead of divide by 255 */
 			alpha = (a * (*src)) / 255;
 			dst[0] = (dst[0] * (255 - alpha) + r * alpha) / 255;
@@ -692,11 +865,28 @@ gog_renderer_pixbuf_draw_text (GogRenderer *rend, char const *text,
 			dst[2] = (dst[2] * (255 - alpha) + b * alpha) / 255;
 			dst[3] = (dst[3] * (255 - alpha) + a * alpha) / 255;
 		}
-		dst += prend->rowstride - w*4;
-		src += ft_bitmap.pitch - w;
+		dst += interrow;
+		src += ft_bitmap.pitch - ft_w;
 	}
-
 	g_free (ft_bitmap.buffer);
+
+	if (rotation_type == GO_ROTATE_FREE) 
+		pango_context_set_matrix (pango_layout_get_context (layout), &old_matrix);
+}
+
+static void
+gog_renderer_pixbuf_get_text_OBR (GogRenderer *rend,
+				  char const *text, GOGeometryOBR *obr)
+{
+	PangoRectangle	 logical;
+	PangoLayout	*layout;
+	
+	layout = gog_renderer_pixbuf_get_pango_layout ((GogRendererPixbuf *) rend);
+	pango_layout_set_text (layout, text, -1);
+	pango_layout_get_extents (layout, NULL, &logical);
+
+	obr->w = ((double) logical.width + (double) PANGO_SCALE / 2.0) / (double) PANGO_SCALE;
+	obr->h = ((double) logical.height + (double) PANGO_SCALE / 2.0) /(double) PANGO_SCALE;
 }
 
 static void
@@ -729,21 +919,6 @@ gog_renderer_pixbuf_draw_marker (GogRenderer *rend, double x, double y)
 				      255);
 }
 
-static void
-gog_renderer_pixbuf_measure_text (GogRenderer *rend,
-				  char const *text, GogViewRequisition *size)
-{
-	PangoRectangle  logical;
-	PangoLayout    *layout;
-	
-	layout = gog_renderer_pixbuf_get_pango_layout ((GogRendererPixbuf *) rend);
-	pango_layout_set_text (layout, text, -1);
-	pango_layout_get_pixel_extents (layout, NULL, &logical);
-
-	size->w = logical.width;
-	size->h = logical.height;
-}
-
 static void
 gog_renderer_pixbuf_pop_style (GogRenderer *rend)
 {
@@ -777,8 +952,8 @@ gog_renderer_pixbuf_class_init (GogRendererClass *rend_klass)
 	gobject_klass->finalize		= gog_renderer_pixbuf_finalize;
 	rend_klass->push_style		= gog_renderer_pixbuf_push_style;
 	rend_klass->pop_style		= gog_renderer_pixbuf_pop_style;
-	rend_klass->clip_push  		= gog_renderer_pixbuf_clip_push;
-	rend_klass->clip_pop     	= gog_renderer_pixbuf_clip_pop;
+	rend_klass->push_clip  		= gog_renderer_pixbuf_push_clip;
+	rend_klass->pop_clip    	= gog_renderer_pixbuf_pop_clip;
 	rend_klass->sharp_path		= gog_renderer_pixbuf_sharp_path;
 	rend_klass->draw_path	  	= gog_renderer_pixbuf_draw_path;
 	rend_klass->draw_polygon  	= gog_renderer_pixbuf_draw_polygon;
@@ -786,7 +961,7 @@ gog_renderer_pixbuf_class_init (GogRendererClass *rend_klass)
 	rend_klass->draw_bezier_polygon = gog_renderer_pixbuf_draw_bezier_polygon;
 	rend_klass->draw_text	  	= gog_renderer_pixbuf_draw_text;
 	rend_klass->draw_marker	  	= gog_renderer_pixbuf_draw_marker;
-	rend_klass->measure_text  	= gog_renderer_pixbuf_measure_text;
+	rend_klass->get_text_OBR	= gog_renderer_pixbuf_get_text_OBR;
 	rend_klass->line_size		= gog_renderer_pixbuf_line_size;
 }
 
@@ -875,20 +1050,22 @@ gog_renderer_pixbuf_update (GogRendererPixbuf *prend, int w, int h, double zoom)
 	gboolean redraw = TRUE;
 	GogView *view;
 	GogViewAllocation allocation;
+	GogGraph *graph;
 
 	g_return_val_if_fail (prend != NULL, FALSE);
 	g_return_val_if_fail (prend->base.view != NULL, FALSE);
 
 	view = prend->base.view;
-	gog_graph_force_update (GOG_GRAPH (view->model));
+	graph = GOG_GRAPH (view->model);
+	gog_graph_force_update (graph);
 	allocation.x = allocation.y = 0.;
 	allocation.w = w;
 	allocation.h = h;
 	if (prend->w != w || prend->h != h) {
 		prend->w = w;
 		prend->h = h;
-		prend->base.scale_x = w / prend->base.logical_width_pts;
-		prend->base.scale_y = h / prend->base.logical_height_pts;
+		prend->base.scale_x = w / graph->width;
+		prend->base.scale_y = h / graph->height;
 		prend->base.scale = MIN (prend->base.scale_x, prend->base.scale_y);
 		prend->base.zoom  = zoom;
 		prend->dpi_x = gog_renderer_pt2r_x (&prend->base, GO_IN_TO_PT ((double)1.))
