@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  */
 
@@ -29,7 +29,6 @@
 #include <goffice/utils/go-math.h>
 
 #include <gsf/gsf-impl-utils.h>
-#include <math.h>
 
 /* We need to define an hair line width for the svg and gnome_print renderer. 
  * 0.24 pt is the dot size of a 300 dpi printer, if the plot is printed at scale 1:1 */
@@ -39,8 +38,6 @@ enum {
 	RENDERER_PROP_0,
 	RENDERER_PROP_MODEL,
 	RENDERER_PROP_VIEW,
-	RENDERER_PROP_LOGICAL_WIDTH_PTS,
-	RENDERER_PROP_LOGICAL_HEIGHT_PTS,
 	RENDERER_PROP_ZOOM
 };
 enum {
@@ -105,13 +102,6 @@ gog_renderer_set_property (GObject *obj, guint param_id,
 					   NULL);
 		gog_renderer_request_update (rend);
 		break;
-	case RENDERER_PROP_LOGICAL_WIDTH_PTS:
-		rend->logical_width_pts = g_value_get_double (value);
-		break;
-
-	case RENDERER_PROP_LOGICAL_HEIGHT_PTS:
-		rend->logical_height_pts = g_value_get_double (value);
-		break;
 
 	case RENDERER_PROP_ZOOM:
 		rend->zoom = g_value_get_double (value);
@@ -135,12 +125,6 @@ gog_renderer_get_property (GObject *obj, guint param_id,
 	case RENDERER_PROP_VIEW:
 		g_value_set_object (value, rend->view);
 		break;
-	case RENDERER_PROP_LOGICAL_WIDTH_PTS:
-		g_value_set_double (value, rend->logical_width_pts);
-		break;
-	case RENDERER_PROP_LOGICAL_HEIGHT_PTS:
-		g_value_set_double (value, rend->logical_height_pts);
-		break;
 	case RENDERER_PROP_ZOOM:
 		g_value_set_double (value, rend->zoom);
 		break;
@@ -233,37 +217,38 @@ gog_renderer_pop_style (GogRenderer *rend)
 }
 
 /**
- * gog_renderer_clip_push :
- * @rend : #GogRenderer
- * @region: #GogViewAllocation
+ * gog_renderer_push_clip :
+ * @rend      : #GogRenderer
+ * @clip_path : #ArtVpath array
  *
- * region defines the current clipping region. 
+ * Defines the current clipping region. 
  **/
 void
-gog_renderer_clip_push (GogRenderer *rend, GogViewAllocation const *region)
+gog_renderer_push_clip (GogRenderer *rend, ArtVpath *clip_path)
 {
 	GogRendererClip *clip;
 	GogRendererClass *klass = GOG_RENDERER_GET_CLASS (rend);
 
 	g_return_if_fail (klass != NULL);
+	g_return_if_fail (clip_path != NULL);
 
 	clip = g_new (GogRendererClip, 1);
-	clip->area = *region;
+	clip->path = clip_path;
 
 	rend->clip_stack = g_slist_prepend (rend->clip_stack, clip);
 	rend->cur_clip = clip;
 	
-	(klass->clip_push) (rend, clip);
+	(klass->push_clip) (rend, clip);
 }
 
 /**
- * gog_renderer_clip_pop :
+ * gog_renderer_pop_clip :
  * @rend : #GogRenderer
  *
  * End the current clipping.
  **/
 void
-gog_renderer_clip_pop (GogRenderer *rend)
+gog_renderer_pop_clip (GogRenderer *rend)
 {
 	GogRendererClip *clip;
 	GogRendererClass *klass = GOG_RENDERER_GET_CLASS (rend);
@@ -273,8 +258,9 @@ gog_renderer_clip_pop (GogRenderer *rend)
 
 	clip = (GogRendererClip *) rend->clip_stack->data;
 
-	(klass->clip_pop) (rend, clip);
+	(klass->pop_clip) (rend, clip);
 
+	g_free (clip->path);
 	g_free (clip);
 	rend->clip_stack = g_slist_delete_link (rend->clip_stack, rend->clip_stack);
 
@@ -284,21 +270,49 @@ gog_renderer_clip_pop (GogRenderer *rend)
 		rend->cur_clip = NULL;
 }
 
+/**
+ * gog_renderer_get_rectangle_vpath :
+ * @x      : upper left corner
+ * @y      : upper left corner
+ * @width  : rectangle width
+ * @height : rectangle height
+ *
+ * a utility routine to build a rectangle path.
+ **/
+ArtVpath * 
+gog_renderer_get_rectangle_vpath (GogViewAllocation const *rect)
+{
+	ArtVpath *path;
+
+	path = g_new (ArtVpath, 6);
+
+	path[0].x = path[3].x = path[4].x = rect->x;
+	path[1].x = path[2].x = rect->x + rect->w;
+	path[0].y = path[1].y = path[4].y = rect->y;
+	path[2].y = path[3].y = rect->y + rect->h;
+	path[0].code = ART_MOVETO;
+	path[1].code = path[2].code = path[3].code = path[4].code = ART_LINETO;
+	path[5].code = ART_END;
+
+	return path;
+}
+
 /**
  * gog_renderer_get_ring_wedge_vpath :
- * @cx : center x coordinate
- * @cy : center y coordinate
- * @rx : x radius
- * @ry : y radius
+ * @x : center x coordinate
+ * @y : center y coordinate
+ * @width  : x radius
+ * @height : y radius
  * @th0 : start arc angle
  * @th1 : stop arc angle
  *
  * a utility routine to build a ring wedge path.
  **/
-ArtBpath * gog_renderer_get_ring_wedge_bpath	(double cx, double cy, 
-						 double rx_out, double ry_out,
-						 double rx_in, double ry_in,
-						 double th0, double th1)
+ArtBpath * 
+gog_renderer_get_ring_wedge_bpath (double cx, double cy, 
+				   double rx_out, double ry_out,
+				   double rx_in, double ry_in,
+				   double th0, double th1)
 {
 	ArtBpath *path;
 	double th_arc, th_out, th_in, th_delta, t;
@@ -395,8 +409,7 @@ gog_renderer_draw_ring_wedge (GogRenderer *rend, double cx, double cy,
 			      double rx_out, double ry_out, 
 			      double rx_in, double ry_in,
 			      double th0, double th1,
-			      gboolean narrow,
-			      GogViewAllocation const *bound)
+			      gboolean narrow)
 {
 	GogRendererClass *klass = GOG_RENDERER_GET_CLASS (rend);
 	ArtBpath *path;
@@ -409,9 +422,9 @@ gog_renderer_draw_ring_wedge (GogRenderer *rend, double cx, double cy,
 		return;
 
 	if (rx_in >= 0.0 && ry_in >= 0.0)
-		(klass->draw_bezier_polygon) (rend, path, narrow, bound);
+		(klass->draw_bezier_polygon) (rend, path, narrow);
 	else
-		(klass->draw_bezier_path) (rend, path, bound);
+		(klass->draw_bezier_path) (rend, path);
 
 	g_free (path);
 }
@@ -420,14 +433,12 @@ gog_renderer_draw_ring_wedge (GogRenderer *rend, double cx, double cy,
  * gog_renderer_draw_sharp_path :
  * @rend : #GogRenderer
  * @path  : #ArtVpath
- * @bound  : #GogViewAllocation optional clip
  *
  * Draws @path using the outline elements of the current style,
  * trying to make line with sharp edge.
  **/
 void
-gog_renderer_draw_sharp_path (GogRenderer *rend, ArtVpath *path,
-			      GogViewAllocation const *bound)
+gog_renderer_draw_sharp_path (GogRenderer *rend, ArtVpath *path)
 {
 	GogRendererClass *klass = GOG_RENDERER_GET_CLASS (rend);
 
@@ -439,27 +450,25 @@ gog_renderer_draw_sharp_path (GogRenderer *rend, ArtVpath *path,
 			gog_renderer_line_size (rend, rend->cur_style->line.width));
 	}
 
-	(klass->draw_path) (rend, path, bound);
+	(klass->draw_path) (rend, path);
 }
 
 /**
  * gog_renderer_draw_path :
  * @rend : #GogRenderer
  * @path  : #ArtVpath
- * @bound  : #GogViewAllocation optional clip
  *
  * Draws @path using the outline elements of the current style.
  **/
 void
-gog_renderer_draw_path (GogRenderer *rend, ArtVpath const *path,
-			GogViewAllocation const *bound)
+gog_renderer_draw_path (GogRenderer *rend, ArtVpath const *path)
 {
 	GogRendererClass *klass = GOG_RENDERER_GET_CLASS (rend);
 
 	g_return_if_fail (klass != NULL);
 	g_return_if_fail (rend->cur_style != NULL);
 
-	(klass->draw_path) (rend, path, bound);
+	(klass->draw_path) (rend, path);
 }
 
 /**
@@ -467,15 +476,13 @@ gog_renderer_draw_path (GogRenderer *rend, ArtVpath const *path,
  * @rend : #GogRenderer
  * @path  : #ArtVpath
  * @narrow : if TRUE skip any outline the current style specifies.
- * @bound  : #GogViewAllocation optional clip
  *
  * Draws @path and fills it with the fill elements of the current style,
  * trying to draw line with sharp edge.
  * If @narrow is false it alos outlines it using the outline elements.
  **/
 void
-gog_renderer_draw_sharp_polygon (GogRenderer *rend, ArtVpath *path, gboolean narrow,
-				 GogViewAllocation const *bound)
+gog_renderer_draw_sharp_polygon (GogRenderer *rend, ArtVpath *path, gboolean narrow)
 {
 	GogRendererClass *klass = GOG_RENDERER_GET_CLASS (rend);
 
@@ -487,7 +494,7 @@ gog_renderer_draw_sharp_polygon (GogRenderer *rend, ArtVpath *path, gboolean nar
 			gog_renderer_line_size (rend, rend->cur_style->outline.width));
 	}
 
-	(klass->draw_polygon) (rend, path, narrow, bound);
+	(klass->draw_polygon) (rend, path, narrow);
 }
 
 /**
@@ -495,21 +502,19 @@ gog_renderer_draw_sharp_polygon (GogRenderer *rend, ArtVpath *path, gboolean nar
  * @rend : #GogRenderer
  * @path  : #ArtVpath
  * @narrow : if TRUE skip any outline the current style specifies.
- * @bound  : #GogViewAllocation optional clip
  *
  * Draws @path and fills it with the fill elements of the current style.
  * If @narrow is false it alos outlines it using the outline elements.
  **/
 void
-gog_renderer_draw_polygon (GogRenderer *rend, ArtVpath const *path, gboolean narrow,
-			   GogViewAllocation const *bound)
+gog_renderer_draw_polygon (GogRenderer *rend, ArtVpath const *path, gboolean narrow)
 {
 	GogRendererClass *klass = GOG_RENDERER_GET_CLASS (rend);
 
 	g_return_if_fail (klass != NULL);
 	g_return_if_fail (rend->cur_style != NULL);
 
-	(klass->draw_polygon) (rend, path, narrow, bound);
+	(klass->draw_polygon) (rend, path, narrow);
 }
 
 
@@ -517,20 +522,18 @@ gog_renderer_draw_polygon (GogRenderer *rend, ArtVpath const *path, gboolean nar
  * gog_renderer_draw_bezier_path :
  * @rend : #GogRenderer
  * @path  : #ArtBpath
- * @bound  : #GogViewAllocation optional clip
  *
  * Draws @path using the outline elements of the current style.
  **/
 void
-gog_renderer_draw_bezier_path (GogRenderer *rend, ArtBpath const *path,
-				GogViewAllocation const *bound)
+gog_renderer_draw_bezier_path (GogRenderer *rend, ArtBpath const *path)
 {
 	GogRendererClass *klass = GOG_RENDERER_GET_CLASS (rend);
 
 	g_return_if_fail (klass != NULL);
 	g_return_if_fail (rend->cur_style != NULL);
 
-	(klass->draw_bezier_path) (rend, path, bound);
+	(klass->draw_bezier_path) (rend, path);
 }
 
 /**
@@ -586,34 +589,53 @@ gog_renderer_draw_marker (GogRenderer *rend, double x, double y)
 }
 
 /**
- * gog_renderer_measure_text :
+ * gog_renderer_get_text_OBR :
  * @rend : #GogRenderer
  * @text : the string to draw
- * @size : #GogViewRequisition to store the size of @text.
+ * @obr : #GOGeometryOBR to store the Object Bounding Rectangle of @text.
  **/
 void
-gog_renderer_measure_text (GogRenderer *rend,
-			   char const *text, GogViewRequisition *size)
+gog_renderer_get_text_OBR (GogRenderer *rend, char const *text, GOGeometryOBR *obr)
 {
 	GogRendererClass *klass = GOG_RENDERER_GET_CLASS (rend);
 
 	g_return_if_fail (klass != NULL);
 	g_return_if_fail (rend->cur_style != NULL);
 	g_return_if_fail (text != NULL);
+	g_return_if_fail (obr != NULL);
 
+	obr->x = obr->y = 0;
 	if (*text == '\0') {
 		/* Make sure invisible things don't skew size */
-		size->w = size->h = 0;
+		obr->w = obr->h = 0;
+		obr->alpha = 0;
 		return;
 	}
 
-	(klass->measure_text) (rend, text, size);
+	(klass->get_text_OBR) (rend, text, obr);
 
 	/* Make sure invisible things don't skew size */
-	if (size->w == 0)
-		size->h = 0;
-	else if (size->h == 0)
-		size->w = 0;
+	if (obr->w == 0)
+		obr->h = 0;
+	else if (obr->h == 0)
+		obr->w = 0;
+	
+	obr->alpha = rend->cur_style->text_layout.angle * M_PI / 180.0;
+}
+
+/**
+ * gog_renderer_get_text_AABR :
+ * @rend : #GogRenderer
+ * @text : the string to draw
+ * @aabr : #GOGeometryAABR to store the Axis Aligned Bounding Rectangle of @text.
+ **/
+void
+gog_renderer_get_text_AABR (GogRenderer *rend, char const *text, GOGeometryAABR *aabr)
+{
+	GOGeometryOBR obr;
+
+	gog_renderer_get_text_OBR (rend, text, &obr);
+	go_geometry_OBR_to_AABR (&obr, aabr);
 }
 
 static void
@@ -651,14 +673,6 @@ gog_renderer_class_init (GogRendererClass *renderer_klass)
 		g_param_spec_object ("view", "view",
 			"the GogView this renderer is displaying",
 			GOG_VIEW_TYPE, G_PARAM_READABLE));
-	g_object_class_install_property (gobject_klass, RENDERER_PROP_LOGICAL_WIDTH_PTS,
-		g_param_spec_double ("logical_width_pts", "Logical Width Pts",
-			"Logical width of the drawing area in pts",
-			0, G_MAXDOUBLE, 0, G_PARAM_READWRITE));
-	g_object_class_install_property (gobject_klass, RENDERER_PROP_LOGICAL_HEIGHT_PTS,
-		g_param_spec_double ("logical_height_pts", "Logical Height Pts",
-			"Logical height of the drawing area in pts",
-			0, G_MAXDOUBLE, 0, G_PARAM_READWRITE));
 	g_object_class_install_property (gobject_klass, RENDERER_PROP_ZOOM,
 		g_param_spec_double ("zoom", "zoom Height Pts",
 			"global scale factor",
@@ -686,8 +700,6 @@ gog_renderer_init (GogRenderer *rend)
 	rend->cur_style    = NULL;
 	rend->style_stack  = NULL;
 	rend->zoom = rend->scale = rend->scale_x = rend->scale_y = 1.;
-	rend->logical_width_pts    = GO_CM_TO_PT ((double)12);
-	rend->logical_height_pts   = GO_CM_TO_PT ((double)8);
 	rend->font_watcher = g_cclosure_new_swap (G_CALLBACK (cb_font_removed),
 		rend, NULL);
 	go_font_cache_register (rend->font_watcher);
@@ -701,13 +713,11 @@ GSF_CLASS (GogRenderer, gog_renderer,
  * gog_renderer_draw_rectangle :
  * @renderer : #GogRenderer
  * @rect : #GogViewAllocation
- * @bound  : #GogViewAllocation optional clip
  *
  * A utility routine to build a vpath in @rect.
  **/
 static void
-draw_rectangle (GogRenderer *rend, GogViewAllocation const *rect,
-		GogViewAllocation const *bound, gboolean sharp)
+draw_rectangle (GogRenderer *rend, GogViewAllocation const *rect, gboolean sharp)
 {
 	gboolean const narrow = (rect->w < 3.) || (rect->h < 3.);
 	double o, o_2;
@@ -730,23 +740,21 @@ draw_rectangle (GogRenderer *rend, GogViewAllocation const *rect,
 	path[1].y = path[2].y = path[0].y + rect->h - o; 
 
 	if (sharp)
-		gog_renderer_draw_sharp_polygon (rend, path, narrow, bound);
+		gog_renderer_draw_sharp_polygon (rend, path, narrow);
 	else
-		gog_renderer_draw_polygon (rend, path, narrow, bound);
+		gog_renderer_draw_polygon (rend, path, narrow);
 }
 
 void
-gog_renderer_draw_sharp_rectangle (GogRenderer *rend, GogViewAllocation const *rect,
-				   GogViewAllocation const *bound)
+gog_renderer_draw_sharp_rectangle (GogRenderer *rend, GogViewAllocation const *rect)
 {
-	draw_rectangle (rend, rect, bound, TRUE);
+	draw_rectangle (rend, rect, TRUE);
 }
 
 void
-gog_renderer_draw_rectangle (GogRenderer *rend, GogViewAllocation const *rect,
-				   GogViewAllocation const *bound)
+gog_renderer_draw_rectangle (GogRenderer *rend, GogViewAllocation const *rect)
 {
-	draw_rectangle (rend, rect, bound, FALSE);
+	draw_rectangle (rend, rect, FALSE);
 }
 
 double
