@@ -20,8 +20,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
+ * USA.
  */
 
 #include <goffice/goffice-config.h>
@@ -37,7 +37,7 @@
 #include <gtk/gtkvbox.h>
 #include <gtk/gtkframe.h>
 #include <gdk/gdkkeysyms.h>
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 
 #include <gsf/gsf-impl-utils.h>
 
